/*
 *
 * this is a test av for ZF4
 * there's no way to specify search mask. anyway, RA does search for *.* ;-)

 * it is Av's task to check whether its Virbase is corrupt.

 * (c) 2001 by Sassa
 *   _____ Apiary Ink
 * @()(_)
 * /\\
 */

import java.io.*;
import java.util.Vector;

import utils.av.*;
import utils.tools.*;

public class Av {
	static final String HELP_SCREEN = "Sample AV for ZF4. (c) 2001 by Sassa\nusage: Av [-doprs] [<paths and files, separated by space>]\n\td Do not automatically delete unrecoverable files\n\to Print OK messages\n\tp Print protocol\n\tr Recurse directories\n\ts Scan only\n\n";

	static final String HEALTHY_STR = "OK";
	static final String SUSPECT_STR = "suspicious";
	static final String DETECTED_STR = "detected";
	static final String UNRECOVERABLE_STR = "unrecoverable";
	static final String HEALED_STR = "healed";

	static final String SCAN_WARNING = "WARNING: scan only mode. the entity has not been changed";


		// for debugging purposes only it is directed to System.out
		// might be null
	static PrintStream protocol = System.out;

	// flags
	static boolean printOk = false;		// prints "OK" messages
	static boolean deleteUnrecoverable = true; // autodelete unrecoverable
	static boolean recurseDirectories = false; // recurse subdirectories
	static boolean saveCured = true;	// save all cured entities; is false in "Scan only" mode

	static final String VIRBASE_FILE = "virbase.dat";
	static final String PROTOCOL_FILE = "protocol.txt";


	static final String GeneralPractitioner = "GP"; // name of the first doctor to visit
			// update this class, if the execution tree adds node from above

	static Vector hospital_staff = new Vector(); // collection of doctors already attended
		// the rest of them might be present, but still asleep on a hard drive
		// HashTable could be of use as well...

	public static void main(String args[]){
	  try{
		if (args.length!=0){
			if (args[0].charAt(0)=='-'){
				for (int i=1; i<args[0].length(); i++){
					switch (args[0].charAt(i)){
					case 'o': printOk=true;
						break;
					case 'd': deleteUnrecoverable=false;
						break;
					case 'r': recurseDirectories=true;
						break;
					case 'p': protocol=new PrintStream(new FileOutputStream(PROTOCOL_FILE));
						break;
					case 's': saveCured=false;
						break;
					default:
						System.out.println(HELP_SCREEN);
						return;
					}
				}

				args[0]=null;
			}
		}

		FileInputStream fi = new FileInputStream(VIRBASE_FILE);

		ZemskijFershal.environment = new byte[fi.available()];
		fi.read(ZemskijFershal.environment);


		// at this point only paths for healing must be left in args
		File roots[] = new File[args.length];
		if (args.length==0){
			roots = File.listRoots();
		}else{
			int i, j;
			for (i=0, j=0; j<args.length; j++){
				if (args[j]!=null) roots[i++] = new File(args[j]);
			}

			for (; i<j; i++){
				roots[i]=null; // set the rest to null
			}
		}

		if (roots!=null){
			for (int i=0; i<roots.length; i++){
				cure(roots[i], true);
			}
		}
	  }catch (FileNotFoundException ffex){
		System.out.println("the required file was not found: "+ffex.getMessage());
	  }catch (Exception ex){
		ex.printStackTrace();
	  }
	}

	static void cure(File file, boolean rd){
	   if (file==null) return;

	   try{
		if (file.isDirectory()){
		  if (rd){
			File dirlist[] = file.listFiles();

			if (dirlist!=null){
				for (int i=0; i<dirlist.length; i++){
					cure(dirlist[i], recurseDirectories);
				}
			}
		  }

		  return;
		}

		if (!file.canRead()){
			throw new AVException("cannot read file "+file.getPath());
		}

		byte [] bt;
		try{
		  FileInputStream fis = new FileInputStream(file.getPath());

		  bt = new byte[fis.available()];
		  fis.read(bt);
		  fis.close();
		}catch(FileNotFoundException fex){
		  return;
		}catch(IOException iex){
		  throw new AVException("error reading file: "+file.getPath());
		}

		Result result = new Result(null, GeneralPractitioner, Result.HEALTHY, bt, false, null);
		String verdict;
		String tab=file.getPath();

		while (result.consult!=null){
			result = call("utils.tools."+result.consult, result);

			if (protocol!=null){
				
				switch (result.diagnosis){
				case Result.HEALTHY: if (result.consult!=null || !printOk) continue;
							verdict=HEALTHY_STR;
							break;
				case Result.SUSPICIOUS: verdict=SUSPECT_STR;
							break;
				case Result.DETECTED: verdict=DETECTED_STR;
							break;
				case Result.UNRECOVERABLE: verdict=UNRECOVERABLE_STR;
							break;
				case Result.HEALED: verdict=HEALED_STR;
							break;
				case Result.NOCHANGES: 
				default:
							verdict="";
							break;
				}

				if (result.verdict!=null && !result.verdict.equals("")){
					verdict= verdict+" - "+result.verdict;
				}

				protocol.println(tab+" : "+verdict);

				tab="\t\t";
			}
		}

		if (result.diagnosis==Result.UNRECOVERABLE && result.save && deleteUnrecoverable && saveCured){
			// it might be reasonable to ask for deleting, if deleteUnrecoverable is false.
			// but in this case it can work in a batch mode.
			file.delete();
		}

		if (result.diagnosis!=Result.UNRECOVERABLE && result.save && saveCured){
			if (!file.canWrite()){
				throw new AVException(file.getPath()+" is not allowed to modify. Change its access attributes or remove it.");
			}

			try{
			  FileOutputStream fos = new FileOutputStream(file.getPath());
			  fos.write(result.file);
			}catch (FileNotFoundException fex){
			  throw new AVException("cannot find my file again 8-) : "+file.getPath());
			}catch(IOException iex){
			  throw new AVException("error writing file: "+file.getPath());
			}
		}

		if (!saveCured && result.diagnosis!=Result.HEALTHY && protocol!=null){
			protocol.println(SCAN_WARNING);
		}
	  }catch (AVException avex){
		if (protocol!=null){
			protocol.println(avex.getMessage());
		}
	  }
	}

	static Result call(String doctor, Result prev) throws AVException{
	  try{
		ZemskijFershal zf = null;

		// search within hospital_staff aleady awakened
		for (int it=0; it<hospital_staff.size(); it++){
			if (hospital_staff.get(it).getClass().getName().equals(doctor)){
				zf = (ZemskijFershal)(hospital_staff.get(it));
				break;
			}
		}

		if (zf==null){
			// of course, at this point an integrity validation check should be performed
			zf = (ZemskijFershal)( ClassLoader.getSystemClassLoader().loadClass(doctor).newInstance() );

			hospital_staff.add(zf);
		}

		return zf.apply(prev);

	  }catch (ClassNotFoundException cex){
		throw new AVException("cannot find the required module: "+doctor+".class");
	  }catch (IllegalAccessException iaex){
		throw new AVException("need rights to access the required module: "+doctor+".class");
	  }catch (InstantiationException iex){
		throw new AVException("cannot instantiate the required module: "+doctor+".class");
	  }
	}
}



